/*
 * Copyright CERN 2013
 * Author: Federico Vaga <federico.vaga@gmail.com>
 */

#ifndef ADC_LIB_H_
#define ADC_LIB_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stdlib.h>
#include <sys/time.h>

/* Error codes start from 1024 to void conflicting with libc codes */
#define __ADC_ERRNO_START 1024
#define ADC_ENOP		1024
#define ADC_ENOCAP		1025
#define ADC_ENOCFG		1026
#define ADC_ENOGET		1027
#define ADC_ENOSET		1028
#define ADC_ENOCHAN		1029
#define ADC_ENOMASK		1030
#define ADC_EDISABLED		1031

struct adc_dev;

extern const char *adc_board_name[];

/**
 * Enumerate all supported boards
 */
enum adc_supported_boards {
	FMCADC_100MS_4CH_14BIT = 0,
	ADC_ZIOFAKE,
	ADC_GENERICFAKE,
	__ADC_SUPPORTED_BOARDS_LAST_INDEX,
};

/**
 * The buffer hosts data and metadata, plus informative fields
 */
struct adc_buffer {
	void *data; /**< container for all the samples */
	void *metadata; /**< description of the acquisition */
	int samplesize; /**< size of a single sample in Byte */
	int nsamples; /**< number of samples in the buffer */
	struct adc_dev *dev; /**< the device used for the acquisition */
	void *mapaddr; /**< mmap address */
	unsigned long maplen; /**< mmap length */
	unsigned long flags; /**< internal to the library */
};

/**
 * Timestamp descriptor. This is exactly the zio_timestamp, but there is
 * no depency on zio here
 */
struct adc_timestamp {
	uint64_t secs; /**< number of seconds */
	uint64_t ticks; /**< number of coarse sub-seconds unit */
	uint64_t bins; /**< high resolution sub-seconds unit */
};

/* The following enum can be use to se the mask of valid configurations */
enum adc_configuration_trigger {
	ADC_CONF_TRG_SOURCE = 0, /**< source of triggers */
	ADC_CONF_TRG_SOURCE_CHAN,  /**< channel number when the source is an
				    internal channel */
	ADC_CONF_TRG_THRESHOLD,  /**< trigger threshold */
	ADC_CONF_TRG_POLARITY,  /**< trigger polarity */
	ADC_CONF_TRG_DELAY,  /**< acquisition delay after trigger*/
	ADC_CONF_TRG_THRESHOLD_FILTER,  /**< */
	__ADC_CONF_TRG_ATTRIBUTE_LAST_INDEX,
};
enum adc_configuration_acquisition {
	ADC_CONF_ACQ_N_SHOTS = 0,  /**< number of shots during the acquisition */
	ADC_CONF_ACQ_POST_SAMP,  /**< number of samples after the trigger */
	ADC_CONF_ACQ_PRE_SAMP,  /**< number of sample before the trigger */
	ADC_CONF_ACQ_DECIMATION,  /**< Acquisition sample decimation*/
	ADC_CONF_ACQ_FREQ_HZ,  /**< Acquisition frequency */
	ADC_CONF_ACQ_N_BITS,  /**< Number of bit per sample */
	__ADC_CONF_ACQ_ATTRIBUTE_LAST_INDEX,  /**< */
};
enum adc_configuration_channel {
	ADC_CONF_CHN_RANGE = 0,  /**< Volt range */
	ADC_CONF_CHN_TERMINATION,  /**< channel Termination */
	ADC_CONF_CHN_OFFSET,  /**< signal offset */
	ADC_CONF_CHN_SATURATION,  /**< Saturation value */
	__ADC_CONF_CHN_ATTRIBUTE_LAST_INDEX,
};
enum adc_board_status {
	ADC_CONF_BRD_STATUS = 0,  /**< Board status*/
	ADC_CONF_BRD_MAX_FREQ_HZ,  /**< Maximum frequency in HZ */
	ADC_CONF_BRD_MIN_FREQ_HZ,  /**< Minimum frequency in HZ */
	ADC_CONF_BRD_STATE_MACHINE_STATUS,  /**< Acquisition state machine
					       status*/
	ADC_CONF_BRD_N_CHAN,  /**< Number of acquisition channel */
	ADC_CONF_UTC_TIMING_BASE_S,  /**< Board internal time: seconds */
	ADC_CONF_UTC_TIMING_BASE_T,  /**< Board internal time: coarse
					sub-seconds */
	ADC_CONF_UTC_TIMING_BASE_B,  /**< Board internal time: fine
					sub-seconds*/
	__ADC_CONF_BRD_ATTRIBUTE_LAST_INDEX,
};
enum adc_configuration_type {
	ADC_CONF_TYPE_TRG = 0,	/**< Trigger */
	ADC_CONF_TYPE_ACQ, /**< Acquisition */
	ADC_CONF_TYPE_CHN, /**< Channel */
	ADC_CONT_TYPE_BRD, /**< Board */
	__ADC_CONF_TYPE_LAST_INDEX,
};


#define __ADC_CONF_LEN 64 /* number of allocated items in each structure */
/**
 * ADC configuration descriptor.
 */
struct adc_conf {
	enum adc_configuration_type type; /**< configuration type */
	uint32_t dev_type; /**< device type */
	uint32_t route_to; /**< internal route to a particular sub-device
			      (e.g. a channel) */
	uint32_t flags; /* how to identify invalid? */
	uint64_t mask;  /**< capabilities mask, 1 when the correspondent
			   value in value[] is valid */
	uint32_t value[__ADC_CONF_LEN];  /**< array of configuration value.
					    The position in the array
					    correspond to a specific
					    capability */
};


/**
 * Flags used in open/acq/config -- note: low-bits are used by lib-int.h
 */
#define ADC_F_USERMASK	0xffff0000
#define ADC_F_FLUSH	0x00010000 /**< Flush the buffer on open */
#define ADC_F_VERBOSE	0x00020000 /**< be verbose during the configuration */

/**
 * @defgroup dev Basic
 * Basic library functions
 * @{
 */
extern int adc_init(void);
extern void adc_exit(void);
extern char *adc_strerror(int errnum);
extern char *adc_get_driver_type(struct adc_dev *dev);
extern struct adc_dev *adc_open(char *name, unsigned int dev_id,
				      unsigned long totalsamples,
				      unsigned int nbuffer,
				      unsigned long flags);
extern struct adc_dev *adc_open_by_lun(char *name, int lun,
					     unsigned long totalsamples,
					     unsigned int nbuffer,
					     unsigned long flags);
extern int adc_close(struct adc_dev *dev);
/**@}*/


/**
 * @defgroup acq Acquisition
 * Acquisition functions
 * @{
 */
extern int adc_acq_start(struct adc_dev *dev, unsigned int flags,
			    struct timeval *timeout);
extern int adc_acq_poll(struct adc_dev *dev, unsigned int flags,
			    struct timeval *timeout);
extern int adc_acq_stop(struct adc_dev *dev, unsigned int flags);
/**@}*/

/**
 * @defgroup conf Configuration
 * Device configuration functions
 * @{
 */
static inline void adc_set_conf_mask(struct adc_conf *conf,
				        unsigned int conf_index)
{
	conf->mask |= (1LL << conf_index);
}


/**
 * It assigns a configuration item, and its mask
 * @param[in] conf configuration structure where apply this operation
 * @param[in] conf_index configuration value to set (see configuration enum)
 * @param[in] val value to set
 */
static inline void adc_set_conf(struct adc_conf *conf,
				   unsigned int conf_index, uint32_t val)
{
	conf->value[conf_index] = val;
	adc_set_conf_mask(conf, conf_index);
}


/**
 * It retrieve a configuration value from the configuration descriptor
 * @param[in] conf configuration structure where apply this operation
 * @param[in] conf_index configuration value to set (see configuration enum)
 * @param[out] val value read
 * @return 0 in success, -1 on error
 */
static inline int adc_get_conf(struct adc_conf *conf,
				  unsigned int conf_index,
				  uint32_t *val)
{
	if (conf->mask & (1LL << conf_index)) {
		*val = conf->value[conf_index];
		return 0;
	} else {
		return -1;
	}
}
extern int adc_reset_conf(struct adc_dev *dev, unsigned int flags,
			       struct adc_conf *conf);
extern int adc_apply_config(struct adc_dev *dev, unsigned int flags,
			       struct adc_conf *conf);
extern int adc_retrieve_config(struct adc_dev *dev,
				 struct adc_conf *conf);
extern int adc_get_param(struct adc_dev *dev, char *name,
			    char *sptr, int *iptr);
extern int adc_set_param(struct adc_dev *dev, char *name,
			    char *sptr, int *iptr);
/**@}*/

/**
 * @defgroup buf Buffer
 * Data management functions
 * @{
 */
extern struct adc_buffer *adc_request_buffer(struct adc_dev *dev,
					     int nsamples,
					     void *(*alloc_fn)(size_t),
					     unsigned int flags);
extern int adc_fill_buffer(struct adc_dev *dev,
			   struct adc_buffer *buf,
			   unsigned int flags,
			   struct timeval *timeout);
extern struct adc_timestamp *adc_tstamp_buffer(struct adc_buffer *buf,
					       struct adc_timestamp *);
extern int adc_release_buffer(struct adc_dev *dev,
			      struct adc_buffer *buf,
			      void (*free_fn)(void *));
/**@}*/

#ifdef __cplusplus
}
#endif

#endif /* ADC_LIB_H_ */
