/*
 * Routing public functions to device-specific code
 *
 * Copyright (C) 2013 CERN (www.cern.ch)
 * Author: Federico Vaga <federico.vaga@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * version 2 as published by the Free Software Foundation or, at your
 * option, any later version.
 */

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "adc-lib.h"
#include "adc-lib-int.h"


/**
 * It starts an acquisition on the given device
 * @param[in] dev ADC device token
 * @param[in] flags flags to control the acquisition options
 * @param[in] timeout time to wait to conclude the acquisition. When 0,
 *            the acquisition will run in backgroun and you can
 *            use adc_acq_poll() to check for data.
 * @return 0 on success, -1 on error and errno is set appropriately
 */
int adc_acq_start(struct adc_dev *dev,
		  unsigned int flags,
		  struct timeval *timeout)
{
	struct adc_gid *g = (void *)dev;
	const struct adc_board_type *b = g->board;

	return b->adc_op->acq_start(dev, flags, timeout);
}


/**
 * It polls the given device to check if data is available
 * @param[in] dev ADC device token
 * @param[in] flags flags to specify how to poll
 * @return 0 on success, -1 on error and errno is set appropriately
 */
int adc_acq_poll(struct adc_dev *dev, unsigned int flags,
		    struct timeval *timeout)
{
	struct adc_gid *g = (void *)dev;
	const struct adc_board_type *b = g->board;

	return b->adc_op->acq_poll(dev, flags, timeout);
}


/**
 * It stops the acquisition on the given device
 * @param[in] dev ADC device token
 * @param[in] flags flags to specify how to stop
 * @return 0 on success, -1 on error and errno is set appropriately
 */
int adc_acq_stop(struct adc_dev *dev, unsigned int flags)
{
	struct adc_gid *g = (void *)dev;
	const struct adc_board_type *b = g->board;

	return b->adc_op->acq_stop(dev, flags);
}


/**
 * It applies the given configuration to the device
 * @param[in] dev ADC device token
 * @param[in] flags
 * @param[in] conf configuration to apply
 * @return 0 on success, -1 on error and errno is set appropriately
 */
int adc_apply_config(struct adc_dev *dev, unsigned int flags,
			struct adc_conf *conf)
{
	struct adc_gid *g = (void *)dev;
	const struct adc_board_type *b = g->board;
	uint64_t cap_mask;

	if (!conf->mask) {
		errno = ADC_ENOMASK;
		return -1; /* Nothing to do */
	}
	cap_mask = b->capabilities[conf->type] & conf->mask;
	if (cap_mask != conf->mask) {
		/* Unsupported capabilities */
		errno = ADC_ENOCAP;
		return -1;
	}
	return b->adc_op->apply_config(dev, flags, conf);
}


/**
 * It retrieve the current device configuration
 * @param[in] dev ADC device token
 * @param[in,out] conf configuration descriptor to fill with configuration
 *                values.
 * @return 0 on success, -1 on error and errno is set appropriately
 */
int adc_retrieve_config(struct adc_dev *dev, struct adc_conf *conf)
{
	struct adc_gid *g = (void *)dev;
	const struct adc_board_type *b = g->board;
	uint64_t cap_mask;

	if (!conf->mask) {
		errno = ADC_ENOMASK;
		return -1; /* Nothing to do */
	}
	cap_mask = b->capabilities[conf->type] & conf->mask;
	if (cap_mask != conf->mask) {
		/* Unsupported capabilities */
		errno = ADC_ENOCAP;
		return -1;
	}
	return b->adc_op->retrieve_config(dev, conf);
}


/**
 * It get a single parameter from the device
 * @param[in] dev ADC device token
 * @param[in] name parameter name to set
 * @param[out] sptr a string value (higher priority than iptr)
 * @param[out] iptr an integer value
 * @return 0 on success, -1 on error and errno is set appropriately
 */
int adc_get_param(struct adc_dev *dev, char *name,
		     char *sptr, int *iptr)
{
	struct adc_gid *g = (void *)dev;
	const struct adc_board_type *b = g->board;

	return b->adc_op->get_param(dev, name, sptr, iptr);
}


/**
 * It set a single parameter to the device
 * @param[in] dev ADC device token
 * @param[in] name parameter name to set
 * @param[in] sptr a string value (higher priority than iptr)
 * @param[in] iptr an integer value
 * @return 0 on success, -1 on error and errno is set appropriately
 */
int adc_set_param(struct adc_dev *dev, char *name,
		     char *sptr, int *iptr)
{
	struct adc_gid *g = (void *)dev;
	const struct adc_board_type *b = g->board;

	return b->adc_op->set_param(dev, name, sptr, iptr);
}


/**
 * It requests a buffer where store data from the acquisition
 * @param[in] dev ADC device token
 * @param[in] nsamples size of the buffer in number of samples
 * @param[in] alloc when set it will use the given function to allocate
 *            the buffer
 * @param[in] flags flags to control the buffer request
 * @return 0 on success, -1 on error and errno is set appropriately
 */
struct adc_buffer *adc_request_buffer(struct adc_dev *dev,
					    int nsamples,
					    void *(*alloc)(size_t),
					    unsigned int flags)
{
	struct adc_gid *g = (void *)dev;
	const struct adc_board_type *b = g->board;

	return b->adc_op->request_buffer(dev, nsamples, alloc, flags);
}


/**
 * It ask the library to get the data from the device and fill the
 * given buffer.
 * @param[in] dev ADC device token
 * @param[in] buf the buffer to fill
 * @param[in] flags used to control how to fill the buffer
 * @param[in] timeout maximum time to fill the buffer
 * @return 0 on success, -1 on error and errno is set appropriately
 */
int adc_fill_buffer(struct adc_dev *dev,
		       struct adc_buffer *buf,
		       unsigned int flags,
		       struct timeval *timeout)
{
	struct adc_gid *g = (void *)dev;
	const struct adc_board_type *b = g->board;

	return b->adc_op->fill_buffer(dev, buf, flags, timeout);
}


/**
 * It gets the timestamp of a given buffer
 * @param[in] dev ADC device token
 * @param[out] ts the buffer timestamp
 * @return the buffer timestamp
 */
struct adc_timestamp *adc_tstamp_buffer(struct adc_buffer *buf,
					      struct adc_timestamp *ts)
{
	struct adc_gid *g = (void *)buf->dev;
	const struct adc_board_type *b = g->board;

	return b->adc_op->tstamp_buffer(buf, ts);
}


/**
 * It frees a given buffer
 * @param[in] dev ADC device token
 * @return 0 on success, -1 on error and errno is set appropriately
 */
int adc_release_buffer(struct adc_dev *dev, struct adc_buffer *buf,
			  void (*free)(void *))
{
	struct adc_gid *g = (void *)dev;
	const struct adc_board_type *b = g->board;

	if (!buf)
		return 0;

	return b->adc_op->release_buffer(dev, buf, free);
}
